# Author: Acer Zhang
# Datetime: 2021/9/8 
# Copyright belongs to the author.
# Please indicate the source for reprinting.

from qwebsite.flags import *

print("当前操作系统：", OS_FLAG)

__version__ = "0.4a1"
